****************************************************************************************
* GXBORDER.PRG	: GenScrnX Driver - replacement for SET READBORDER ON (Win)
* Author		: Julian Ransom
* CIS Id		: 100415,2675
* Version		: 1.00
* Copyright		: None (Public Domain)
*
* Note:
*   This software is provided "as is" without
*   any warranty expressed or implied. In no event shall
*   the author (Julian Ransom) be liable for any damages.
*
* Description:
*	ReadBorder replacement for Windows.
*	This driver has the same effect as SET READBORDER ON, but is faster
*	and does away with the horrible screen refresh effects.
*	GXBORDER resizes fields and generates boxes in the same way as READBORDER,
*	but does this when you generate the screen.
*
* Usage:
*	To enable the GXBORDER driver for a specific screen,
*	place the following in the Setup Snippet:
* 		*:SCXDRV5 GXBORDER
*
*	To enable the GXBORDER driver for _all_ screens,
*	place the following in your CONFIG.FPW:
* 		_SCXDRV5 = GXBORDER
*
*	... and ensure that GXBorder.PRG is in your path.
*	Note that GXBorder is ignored in Dos.
*
* Optional parameters:
*	To specify the gap between your fields (READBORDER eliminates this gap),
*	use the following in the Setup Snippet:
*		*:GXBDepth <ExpN>
*	The default gap is 1. With READBORDER OFF, fields which just 'touch' each other
*	will look fine in the screen designer, but have a 1-line gap between them
*	in the generated screen. GXBORDER adjusts the height and/or position of your
*	fields to make them fit together neatly.
*
* And finally:
*	Please feel free to send me your comments and suggestions, either
*	on CIS (see above) or the Internet (julian@ransom.demon.co.uk).
*
****************************************************************************************

PRIVATE	fontface, fontsize, lineheight, linewidth,;
		Platform, ObjType, ObjCode, PenSize, PenPat,;
		left, right, height, width,;
		hpos, vpos, DrawnTop, DrawnBot, Gap, recno

#DEFINE	cnRoundErr	.001

Platform	= 'WINDOWS'
ObjType		= 6
ObjCode		= 63
PenSize		= 1
PenPat		= 8

locate for platform=m.platform and objtype=1
m.fontface	= fontface
m.fontsize	= fontsize

m.lineheight= 1/fontmetric(1, m.fontface, m.fontsize)
m.linewidth	= 1/fontmetric(6, m.fontface, m.fontsize)

if atc('GXBDEPTH', setupcode)>0
	Gap	= val(substr(setupcode, atc('GXBDEPTH', setupcode) + 9))
else
	Gap	= 1
endif

wait window str(m.Gap)

* Scan GETs:
scan for platform=m.platform and objtype=15 and objcode=1
	m.Left	= hpos - m.linewidth
	m.Right	= hpos + width
	
	height	= m.lineheight
	
	* Find GETs which the bottom of this GET overlaps:
	m.DrawnBot	= Horizontal(vpos + height, vpos + height, 2)
	if m.DrawnBot=0 and m.Gap>0
		replace height with height + m.gap * m.LineHeight
	endif

	* Find GETs which overlap the TOP of this GET:
	if Horizontal(vpos - m.lineheight, vpos - m.lineheight, 1)=0 and m.Gap>0
		do case
		case m.DrawnBot=0 && No bottom line
			replace vpos with vpos - m.gap * m.LineHeight,;
					height with height + m.gap * m.LineHeight
		case m.DrawnBot=2 && Complete bottom line
			* Move this field and the drawn bottom line up
			m.recno	= recno()
			replace for ObjType=m.ObjType and ObjCode=m.ObjCode;
					and comment='GXBB' + str(m.recno, 5);
					vpos with vpos - m.gap * m.LineHeight
			go m.recno
			replace vpos with vpos - m.gap * m.LineHeight
		endcase
	endif
endscan

scan for platform=m.platform and objtype=15 and objcode=1
	m.Left	= hpos - m.linewidth
	m.Right	= hpos + width
	
	height	= m.lineheight
	width	= m.LineWidth
	
	* Find GETs which share the LHS of this GET
	do Vertical with hpos, m.Left, 1
	
	* Find GETs which share the RHS of this GET
	do Vertical with hpos + width, m.Right, 2
endscan



FUNCTION	Horizontal
PARAMETERS	GetVPos, VPos, TopOrBot
* Handle Horizontal line (Top or Bottom of get)
* GetVPos is the VPos of the original GET
* VPos is the position in which the generated line is to be put
* TopOrBot = 1 for Top, 2 for Bot
PRIVATE		recno, LOverlap, ROverlap,;
			hpos, width, style, drawn
m.recno	= recno()
m.style	= 1

m.LOverlap	= -1
m.ROverlap	= 999999
scan for platform=m.platform and recno()#m.recno and objtype=15 and objcode=1;
		and between(m.GetVPos, vpos - 2 * m.LineHeight - cnRoundErr, vpos + height + cnRoundErr);
		and hpos<=m.Right and hpos + width>=m.Left
	m.LOverlap	= max(m.LOverlap, hpos - m.linewidth)
	m.ROverlap	= min(m.ROverlap, hpos + width)
endscan

drawn	= 1 && Some
do case
case m.LOverlap>-1 and m.LOverlap<=m.Left and m.ROverlap>=m.Right
	* Complete overlap - don't draw
	drawn	= 0
case m.LOverlap>m.Left and m.ROverlap>=m.Right
	* Single line, m.Left to m.LOverlap
	do AddHLine with m.Left - m.LineWidth, m.LOverlap
case m.LOverlap<=m.Left and m.ROverlap<m.Right
	* Single line, m.ROverlap to m.Right
	do AddHLine with m.ROverlap, m.Right + m.LineWidth
case m.LOverlap>m.ROverlap and m.LOverlap>m.Left and m.ROverlap<m.Right
	* One line, m.ROverlap to m.LOverlap
	do AddHLine with m.ROverlap, m.LOverlap
case m.LOverlap>m.Left and m.ROverlap<m.Right
	* Two lines, m.Left to m.LOverlap and m.ROverlap to m.Right
	do AddHLine with m.Left - m.LineWidth, m.LOverlap
	do AddHLine with m.ROverlap, m.Right + m.LineWidth
otherwise
	* Draw complete line, m.Left to m.Right
	do AddHLine with m.Left - m.LineWidth, m.Right + m.LineWidth
	drawn	= 2 && Complete
endcase

go m.recno
return m.Drawn


FUNCTION	AddHLine
PARAMETERS	from, to
PRIVATE		hpos, width
* Add a horizontal line
= INSBLANK()
hpos	= m.from
width	= m.to - m.from
comment	= 'GXB' + iif(m.TopOrBot=1, 'T', 'B') + str(m.recno, 5)
GATHER MEMVAR MEMO


FUNCTION	Vertical
PARAMETERS	GetHPos, HPos, LeftOrRight
* Handle Vertical line (Left or Right of get)
* GetHPos is the HPos of the original GET
* HPos is the position in which the generated line is to be put
* LeftOrRight is 1 for Left, 2 for Right
PRIVATE		recno, LineTop, LineBot,;
			vpos, height, style,;
			penred, penblue, pengreen
m.recno	= recno()
m.style	= 0
m.VPos		= vpos
m.LineTop	= vpos
m.LineBot	= vpos + height
scan for platform=m.platform and recno()#m.recno and objtype=15 and objcode=1;
			and hpos=m.GetHPos;
			and vpos<=m.LineBot + m.Gap * m.LineHeight + cnRoundErr and vpos + height>=m.LineTop - cnRoundErr
	LineTop	= min(m.LineTop, vpos)
 	LineBot	= max(m.LineBot, vpos + height)
endscan

if m.LineTop=m.VPos
	if m.LeftOrRight=1
		= INSBLANK()
		vpos	= m.LineTop
		height	= m.LineBot - m.LineTop
		penred	= 255
		penblue	= 255
		pengreen= 255
		GATHER MEMVAR
		= INSBLANK()
		hpos	= m.hpos - m.LineWidth
		vpos	= m.LineTop
		height	= m.LineBot - m.LineTop
		penred	= 0
		penblue	= 0
		pengreen= 0
		GATHER MEMVAR
	else
		= INSBLANK()
		vpos	= m.LineTop
		height	= m.LineBot - m.LineTop
		penred	= 0
		penblue	= 0
		pengreen= 0
		GATHER MEMVAR
	endif
endif
go m.recno

